/* Index access routines */ 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "utils.h"

#include "ihipdm.h" 

int load_idx_hip_dm (char* ifile,idx_hip_dm* index) 
{
/* Routine to load an index from a file. The structure for the index must
 * already exist as must the file of course
 */
   idx_hip_dm idx;
   FILE *fp;
   int i;

   if ( (fp=fopen(ifile,"r"))==NULL)
   {
	fprintf(stderr,"load_idx_hip_dm unable to open %s \n",ifile);
	return (-1);
   }
   if (index == NULL) 
   {
      fprintf (stderr,"load_idx_hip_dm NULL pointer for index\n");
      return (-1);
   }
   /*printf("Loading index idx_hip_dm from %s.. ",ifile);*/
   for (i=0;i<idx_hip_dm_entries;i++)
   {
	if ( (getnext_idx_hip_dm (fp,&index[i])) != 0)
   	{
	   fprintf(stderr,"load_idx_hip_dm problem with entry %d \n",i);
	   break;
	}
   /*     if  ( (i % 5000) == 0) printf("#");*/
   }
   if ( i < idx_hip_dm_entries) 
   {
	fprintf(stderr,"Premature end of index file \n");
   	return -1;
   }
   /* printf(" done \n"); */
   return 0;
} /* End of load_idx_hip_dm*/

int getnext_idx_hip_dm (FILE* fp, idx_hip_dm* entry)
{
   char buffer[idx_hip_dm_REC_LEN + 2];
   if (fgets((char *)&buffer,idx_hip_dm_REC_LEN+1,fp))
   { 
	char delimiter[3]="|\r\0";
	char *token=strtok((char *)&buffer,(char *)&delimiter); 
	strAsINT(token,&entry->hdm_idx1);
	token=strtok(NULL,delimiter);
	strcpy((char *)&entry->hdm_idx2,token);
	token=strtok(NULL,delimiter);
	strAsINT(token,&entry->hdm_idx3);
   return 0;
   }
   else
        return -1;
} /* End of getnext_idx_hip_dm */

idx_hip_dm* find_idx_hip_dm (INT* key)
{
   static int init=0;
   static idx_hip_dm index[idx_hip_dm_entries];
   int mid,lpos,rpos,found;
   if (init == 0)
   {
        if (load_idx_hip_dm("hip_dm.idx",(idx_hip_dm*)&index) !=0)
             return NULL;
        init=1;
   }

   /*  Binary search */ 
   rpos = key->value; 
   lpos = 0;
   if (rpos > idx_hip_dm_entries) rpos = idx_hip_dm_entries;
   found = 0;
   while (!found && !(lpos==mid && rpos==mid || lpos > rpos))
   {
	mid =  (rpos + lpos) /2;
        found = index[mid].hdm_idx1.value == key->value;
     	if (key->value > index[mid].hdm_idx1.value )
	   lpos = mid+1;
	else
	   if (key->value < index[mid].hdm_idx1.value)
	      rpos = mid;
	/*printf (" %d,%d Betweeen %d,%d and %d,%d Key = %d\n",mid,index[mid].hdm_idx1.value,lpos,index[lpos].hdm_idx1.value,rpos,index[rpos].hdm_idx1.value,key->value);*/
   }
   if (found)
	return &index[mid];
   else
   {
      return NULL;
   }
   
} /* End of find_idx_hip_dm */


